<?php

class Instagram extends Downloader
{
    public static $cookieFile = __DIR__ . '/../../cookies/ig-cookie.txt';
    private $postPage;

    public function fetch($videoUrl)
    {
        $videoUrl = preg_replace('/\/reel\//', '/tv/', $videoUrl);
        $videoUrl = strtok($videoUrl, '?');
        if (substr($videoUrl, -1) != '/') {
            $videoUrl .= '/';
        }
        $this->source = 'instagram';
        preg_match('/https:\/\/www.instagram.com\/reel\/(.*?)\//', $videoUrl, $isReel);
        if (!empty($isReel[1])) {
            $videoUrl = str_replace('/reel/', '/p/', $videoUrl);
            $this->postPage = $this->urlGetContents($videoUrl);
            $data = $this->parsePostPage($this->postPage);
            if (!empty($data)) {
                $this->title = $data['title'];
                $this->thumbnail = $data['thumbnail'];
                $this->medias[] = $data['medias'];
                $this->saveThumbnail();
                $this->thumbnailHotlinkProtection = true;
            }
        } else {
            $this->postPage = $this->urlGetContents($videoUrl);
            if (strpos($this->postPage, 'WatchOnInstagram')) {
                $this->mediaInfoLegacy($videoUrl);
            }
            $data = $this->parsePostPage2($this->postPage);
            if (!empty($data)) {
                if (empty($data['sidecar'])) {
                    $this->title = $data['title'];
                    $this->thumbnail = $data['thumbnail'];
                    $this->medias = $data['medias'];
                } else {
                    if ($data['shortcode_media']['__typename'] == 'GraphSidecar') {
                        $multipleData = $data['shortcode_media']['edge_sidecar_to_children']['edges'];
                        foreach ($multipleData as $media) {
                            $audioAvailable = false;
                            $mediaUrl = null;
                            if ($media['node']['is_video'] == 'true') {
                                if (!empty($media['node']['video_url'])) {
                                    $mediaUrl = $media['node']['video_url'];
                                    $type = 'mp4';
                                    $audioAvailable = true;
                                } else {
                                    $postPage = $this->urlGetContents('https://www.instagram.com/p/' . $media['node']['shortcode']);
                                    $data = $this->parsePostPage($postPage);
                                    if (!empty($data['medias'])) {
                                        $this->medias = array_merge($this->medias, $data['medias']);
                                        continue;
                                    }
                                }
                            } else {
                                $length = count($media['node']['display_resources']);
                                $mediaUrl = $media['node']['display_resources'][$length - 1]['src'];
                                $type = 'jpg';
                            }
                            if ($mediaUrl === null) {
                                continue;
                            }
                            $this->medias[] = new Media($mediaUrl, 'hd', $type, true, $audioAvailable);
                        }
                    } else {
                        if ($data['shortcode_media']['__typename'] == 'GraphVideo') {
                            $this->medias[] = new Media($data['shortcode_media']['video_url'], 'hd', 'mp4', true, true);
                        }
                    }
                }
            }
            $this->title = Helpers::truncateString($this->title);
            $this->thumbnail = html_entity_decode($this->thumbnail);
            $this->saveThumbnail();
            $this->thumbnailHotlinkProtection = true;
        }
    }

    private function parsePostPage($postPage)
    {
        $result = [];
        preg_match_all('/window._sharedData = (.*);<\/script>/', $postPage, $matches);
        if (!empty($matches[1][0])) {
            $data = json_decode($matches[1][0], true);
            if (!empty($data['entry_data']['PostPage'][0]['graphql']['shortcode_media'])) {
                $data = $data['entry_data']['PostPage'][0]['graphql']['shortcode_media'];
                $result['title'] = 'Instagram Video';
                $result['thumbnail'] = html_entity_decode($data['display_url']);
                $result['medias'] = [];
                $result['medias'][] = new Media($data['video_url'], 'hd', 'mp4', true, true);
            }
        } else {
            $result = $this->parsePostPage2($postPage);
        }
        return $result;
    }

    private function parsePostPage2($postPage)
    {
        $result = [];
        preg_match_all('/window\.__additionalDataLoaded\(\'.*\',(.*?)\);/', $postPage, $matches);
        if (isset($matches[1][0]) != '') {
            $data = json_decode($matches[1][0], true);
            $result['medias'] = [];
            //print_r($data);
            if (!empty($data['shortcode_media'])) {
                preg_match_all('/<img class="EmbeddedMediaImage" alt=".*" src="(.*?)"/', $postPage, $matches);
                if (isset($matches[1][0]) != '') {
                    $result['title'] = Helpers::getStringBetween($postPage, '<img class="EmbeddedMediaImage" alt="', '"');
                    $result['thumbnail'] = $matches[1][0];
                    $mediaUrl = html_entity_decode($matches[1][0]);
                    $result['medias'][] = new Media($mediaUrl, 'hd', 'jpg', true, false);
                } else {
                    $result['title'] = $data['shortcode_media']['edge_media_to_caption']['edges'][0]['node']['text'] ?? '';
                    if (empty($result['title']) && isset($data['shortcode_media']['owner']['username']) != '') {
                        $result['title'] = 'Instagram Post from ' . $data['shortcode_media']['owner']['username'];
                    } else {
                        $result['title'] = 'Instagram Post';
                    }
                    $result['thumbnail'] = $data['shortcode_media']['display_resources'][0]['src'];
                    if ($data['shortcode_media']['__typename'] == "GraphImage") {
                        $imagesData = $data['shortcode_media']['display_resources'];
                        $length = count($imagesData);
                        $result['medias'][] = new Media($imagesData[$length - 1]['src'], 'hd', 'jpg', true, false);
                    } else {
                        $result['sidecar'] = true;
                        $result['data'] = $data;
                    }
                }
            } else if (!empty($data['items'])) {
                $result['medias'] = [];
                foreach ($data['items'] as $item) {
                    $result['title'] = $item['caption']['text'] ?? "Instagram Post";
                    if (!empty($item['carousel_media'])) {
                        $result['thumbnail'] = $item['carousel_media'][0]['image_versions2']['candidates'][0]['url'];
                        foreach ($item['carousel_media'] as $carouselMedia) {
                            if (!empty($carouselMedia['video_versions'])) {
                                foreach ($carouselMedia['video_versions'] as $video) {
                                    $result['medias'][] = new Media($video['url'], $video['height'] . 'p', 'mp4', true, true);
                                }
                            }
                            if (!empty($carouselMedia['image_versions2']['candidates'])) {
                                $result['medias'][] = new Media($carouselMedia['image_versions2']['candidates'][0]['url'], 'hd', 'jpg', false, false);
                            }
                        }
                    } else if (!empty($item['video_versions'])) {
                        $result['thumbnail'] = $item['image_versions2']['candidates'][0]['url'];
                        foreach ($item['video_versions'] as $video) {
                            $result['medias'][] = new Media($video['url'], $video['height'] . 'p', 'mp4', true, true);
                        }
                    } else {
                        $result['thumbnail'] = $item['image_versions2']['candidates'][0]['url'];
                        $result['medias'][] = new Media($item['image_versions2']['candidates'][0]['url'], 'hd', 'jpg', false, false);
                    }
                }
            }
        }
        return $result;
    }

    function mediaInfoLegacy($url)
    {
        $this->postPage = $this->urlGetContents($url);
        $this->source = 'instagram';
        $mediaInfo = $this->mediaData($this->postPage);
        if (empty($mediaInfo)) {
            $data = $this->parsePostPage2($this->postPage);
            if (!empty($data['medias'])) {
                $this->title = $data['title'];
                $this->thumbnail = $data['thumbnail'];
                $this->medias[] = $data['medias'];
                $this->saveThumbnail();
                $this->thumbnailHotlinkProtection = true;
            }
        } else {
            $this->title = $this->getTitle($this->postPage);
            $this->thumbnail = $this->getThumbnail($this->postPage);
            $this->saveThumbnail();
            $this->thumbnailHotlinkProtection = true;
            foreach ($mediaInfo['links'] as $link) {
                if (empty($link['type'])) {
                    continue;
                }
                switch ($link['type']) {
                    case 'video':
                        $this->medias[] = new Media($link['url'], 'hd', 'mp4', true, true);
                        break;
                    case 'image':
                        $this->medias[] = new Media($link['url'], 'hd', 'jpg', true, false);
                        break;
                    default:
                        break;
                }
            }
        }
    }

    private function saveThumbnail()
    {
        $id = sha1($this->thumbnail);
        $cache = new Cache('ig-' . $id, 'jpg', $this->urlGetContents($this->thumbnail));
        $this->thumbnail = $cache->url;
    }

    private function urlGetContents($url)
    {
        //return file_get_contents(__DIR__ . '/ig.html');
        $http = new Http($url);
        $http->addCurlOption(CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        if (file_exists(self::$cookieFile)) {
            $http->addCurlOption(CURLOPT_COOKIEFILE, self::$cookieFile);
        }
        $http->run();
        return $http->response;
    }

    function mediaData($postPage)
    {
        preg_match_all("/window.__additionalDataLoaded.'.{5,}',(.*).;/", $postPage, $matches);
        if (!$matches) {
            return false;
        } else {
            preg_match_all("/window.__additionalDataLoaded.'.{5,}',(.*).;/", $postPage, $matches);
            preg_match_all('/<script type="text\/javascript">window._sharedData = (.*?);<\/script>/', $postPage, $output);
            if (isset($matches[1][0]) != '') {
                $json = $matches[1][0];
            } else if (isset($output[1][0]) != '') {
                $json = $output[1][0];
            }
            $data = json_decode($json, true);
            if (!empty($data['entry_data']['PostPage'][0]['graphql'])) {
                $data = $data['entry_data']['PostPage'][0];
            }
            if (empty($data['graphql'])) {
                return false;
            }
            if ($data['graphql']['shortcode_media']['__typename'] == 'GraphImage') {
                $imagesData = $data['graphql']['shortcode_media']['display_resources'];
                $length = count($imagesData);
                $mediaInfo['links'][0]['type'] = 'image';
                $mediaInfo['links'][0]['url'] = $imagesData[$length - 1]['src'];
                $mediaInfo['links'][0]['status'] = 'success';
            } else {
                if ($data['graphql']['shortcode_media']['__typename'] == "GraphSidecar") {
                    $counter = 0;
                    $multipledata = $data['graphql']['shortcode_media']['edge_sidecar_to_children']['edges'];
                    foreach ($multipledata as $media) {
                        if ($media['node']['is_video'] == 'true') {
                            $mediaInfo['links'][$counter]["url"] = $media['node']['video_url'];
                            $mediaInfo['links'][$counter]["type"] = 'video';
                        } else {
                            $length = count($media['node']['display_resources']);
                            $mediaInfo['links'][$counter]["url"] = $media['node']['display_resources'][$length - 1]['src'];
                            $mediaInfo['links'][$counter]["type"] = 'image';
                        }
                        $counter++;
                        $mediaInfo['type'] = 'media';
                    }
                    $mediaInfo['status'] = 'success';
                } else {
                    if ($data['graphql']['shortcode_media']['__typename'] == 'GraphVideo') {
                        $videolink = $data['graphql']['shortcode_media']['video_url'];
                        $mediaInfo['links'][0]['type'] = 'video';
                        $mediaInfo['links'][0]['url'] = $videolink;
                        $mediaInfo['links'][0]['status'] = 'success';
                    } else {
                        $mediaInfo['links']['status'] = 'fail';
                    }
                }
            }
            return $mediaInfo;
        }
    }

    function getThumbnail($postPage)
    {
        preg_match_all("/window.__additionalDataLoaded.'.{5,}',(.*).;/", $postPage, $matches);
        preg_match_all('/<script type="text\/javascript">window._sharedData = (.*?);<\/script>/', $postPage, $output);
        if (isset($matches[1][0]) != '') {
            $json = $matches[1][0];
        } else if (isset($output[1][0]) != '') {
            $json = $output[1][0];
        }
        $data = json_decode($json, true);
        if (!empty($data['entry_data']['PostPage'][0]['graphql'])) {
            $data = $data['entry_data']['PostPage'][0];
        }
        return $data['graphql']['shortcode_media']['display_resources'][0]['src'];
    }

    function getTitle($postPage)
    {
        $title = '';
        if (preg_match_all('@<title>(.*?)</title>@si', $postPage, $match)) {
            $title = $match[1][0];
        }
        return $title;
    }
}